//---------------------------------------------------------------------------
//      DS18S20
//---------------------------------------------------------------------------
#pragma hdrstop
#include "DS18S20.h"
#include "ProgressBar.h"        //   -
#include "MainProgram.h"        //    
#include "ChipFunctions.h"      //      
#include "PortFunctions.h"      //    
#include "CalcCRC.h"            //   CRC
#include "SendMsgFunctions.h"   //     
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
//---   SCRATCHPAD (0x4E) (   1) ----------------------
//---------------------------------------------------------------------------
// output:      Test - ,      (0xFF) + Success
DS18S20_OUTPUT_DATA DS18S20_WriteToScratchpad()
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  DS18S20_OUTPUT_DATA OutputData;       //   

  OutputData.Test=0;
  OutputData.Success=false;             //   

  //  -
  Application->CreateForm(__classid(TLBar),&LBar);      //    -
  LBar->Show();                                         //     

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Write Scratchpad
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x4E;                                  // Write Scratchpad
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //***     ***
  //  TH
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=SelectedChip.Area1[2];                 //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  LBar->ProgressBar1->Position=100*1/2;      //    -
  // TL
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=SelectedChip.Area1[3];                 //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  LBar->ProgressBar1->Position=100*2/2;      //    -
  Sleep(10);
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  LBar->Free();                                         //    -
  OutputData.Success=true;
  return OutputData;          //    Test   
}
//---------------------------------------------------------------------------
//---   SCRATCHPAD (0xBE) (   1) ----------------------
//---------------------------------------------------------------------------
// output:      Test + Success
DS18S20_OUTPUT_DATA DS18S20_ReadFromScratchpad()
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  DS18S20_OUTPUT_DATA OutputData;       //   
  unsigned char CRC8;                   //   CRC8

  OutputData.Test=0;
  OutputData.Success=false;             //  

  //  -
  Application->CreateForm(__classid(TLBar),&LBar);      //    -
  LBar->Show();                                         //     

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Scratchpad-
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0xBE;                                  // Read Scratchpad
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  CRC8=0;                                       //     CRC8
  Sleep(10);
  //***    ***
  for(unsigned int i=0; i<SelectedChip.Area1Size; i++)  //    
  { //   
    wr_buf.Size=1;
    wr_buf.Data[0]=0x62;                                // ReadByte_1W
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
    //   
    rd_buf=ReadFrom();                              // 
    if(!rd_buf.Size) { SendMsgToLogForm(21);          //   
                       LBar->Free();                    //    -
                       ClosePort();
                       return OutputData;
                     }
    SelectedChip.Area1[i]=rd_buf.Data[0];       //      (Area1)
    //  CRC8 (CRC8    ,    ,   )
    for(unsigned int j=0;j<8;j++)
    { CRC8=CalcCRC8(CRC8, rd_buf.Data[0]);                      //   CRC8
      rd_buf.Data[0]>>=1;                                       //   (   )
    }
    LBar->ProgressBar1->Position=100*(i+1)/SelectedChip.Area1Size;      //    -
    Sleep(1);
  }
  //  CRC8 (1 , )
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //   ( CRC8)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21);          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  Sleep(1);
  //   CRC8
  for(unsigned int j=0;j<8;j++)
  { CRC8=CalcCRC8(CRC8, rd_buf.Data[0]);                      //   CRC8
    rd_buf.Data[0]>>=1;                                       //   (   )
  }
  //  
  if(CRC8!=0)   SendMsgToLogForm(14);          //  CRC8   //    -      CRC8
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();                  //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  LBar->Free();                                         //    -
  OutputData.Success=true;
  return OutputData;
}
//---------------------------------------------------------------------------
//---   SCRATCHPAD  EEPROM (0x48) -----------------------------
//---------------------------------------------------------------------------
// output:      Status + Success
DS18S20_OUTPUT_DATA DS18S20_CopyScratchpadToMemory()
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  DS18S20_OUTPUT_DATA OutputData;       //   

  OutputData.Test=0;
  OutputData.Success=false;             //  

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Copy Scratchpad
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x48;                                  // Copy Scratchpad
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(10);
  //         (Test)
  Sleep(1);
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //     (Test)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21);          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.Test=rd_buf.Data[0];
  Sleep(1);
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  OutputData.Success=true;
  return OutputData;            //    Test   
}
//---------------------------------------------------------------------------
//---   EEPROM  SCRATCHPAD (0xB8) -----------------------------
//---------------------------------------------------------------------------
// output:      Status + Success
DS18S20_OUTPUT_DATA DS18S20_RecallE2()
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  DS18S20_OUTPUT_DATA OutputData;       //   

  OutputData.Test=0;
  OutputData.Success=false;             //  

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Recall E2
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0xB8;                                  // Recall E2
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(10);
  //         (Test)
  Sleep(1);
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //     (Test)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21);          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.Test=rd_buf.Data[0];
  Sleep(1);
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  OutputData.Success=true;
  return OutputData;            //    Test   
}
//---------------------------------------------------------------------------
//---    POWER (0xB4) -----------------------------------
//---------------------------------------------------------------------------
// output:      Status + Success
DS18S20_OUTPUT_DATA DS18S20_ReadPowerSupply()
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  DS18S20_OUTPUT_DATA OutputData;       //   

  OutputData.Test=0x55;                 // 0x0 - parasite power, 0xFF - separate power
  OutputData.Success=false;             //  

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Read Power Supply
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0xB4;                                  // Read Power Supply
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //         (Test)
  Sleep(1);
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //     (Test)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21);          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.Test=rd_buf.Data[0];               //    
  Sleep(1);
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  OutputData.Success=true;
  //          
  return OutputData;            //    Test   
}
//---------------------------------------------------------------------------
//---   (0x44) ------------------------------------------
//---------------------------------------------------------------------------
// output:      Status + Success
DS18S20_OUTPUT_DATA DS18S20_ConvertT()
{ WRITE_BUFFER wr_buf;                  //    
  READ_BUFFER rd_buf;                   //    
  DS18S20_OUTPUT_DATA OutputData;       //   

  OutputData.Test=0;
  OutputData.Success=false;             //  

  //  Reset
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  Match ROM
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x55;                                  // Match ROM
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //  8  ROM-
  for(unsigned int i=0; i<8; i++)                       //   
  { wr_buf.Size=2;
    wr_buf.Data[0]=0x61;                                // SendByte_1W
    wr_buf.Data[1]=SelectedChip.ROM[i];                 // ROM Bytes
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                           LBar->Free();          //    -
                           ClosePort();
                           return OutputData;
                         }
    Sleep(1);
  }
  //  Convert Temperature
  wr_buf.Size=2;
  wr_buf.Data[0]=0x61;                                  // SendByte_1W
  wr_buf.Data[1]=0x44;                                  // Convert T
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(10);
  //         (Test)
  Sleep(1);
  wr_buf.Size=1;
  wr_buf.Data[0]=0x62;                                  //   
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  Sleep(1);
  //     (Test)
  rd_buf=ReadFrom();                                // 
  if(!rd_buf.Size) { SendMsgToLogForm(21);          //   
                     LBar->Free();                      //    -
                     ClosePort();
                     return OutputData;
                   }
  OutputData.Test=rd_buf.Data[0];
  Sleep(1);
  //  Reset   
  wr_buf.Size=1;
  wr_buf.Data[0]=0x60;
  if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);          //   
                         LBar->Free();            //    -
                         ClosePort();
                         return OutputData;
                       }
  OutputData.Success=true;
  return OutputData;            //    Test   
}
//---------------------------------------------------------------------------
//---    ---------------------------------------
//---------------------------------------------------------------------------
float DS18S20_DecodeTemperature()
{ float OutputValue;            //   

  //  OutputValue
  OutputValue=(1.0*SelectedChip.Area1[0])/2;            //    LSB
  if(SelectedChip.Area1[1])     OutputValue*=-1.0;      //    MSB
  //   
  OutputValue=OutputValue+0.75-(1.0*SelectedChip.Area1[6])/SelectedChip.Area1[7];
  return OutputValue;
}
//---------------------------------------------------------------------------
//---   ----------------------------------------------------
//---------------------------------------------------------------------------
void DS18S20_DecodeThresholds()
{ unsigned int R1R0, Temp;      //        
  int TH, TL;                   //  
  unsigned char THByte, TLByte; //    
  AnsiString TempStr;           //   

  //   
  TH=0; TL=0; Temp=1;
  THByte=SelectedChip.Area1[2]; TLByte=SelectedChip.Area1[3];
  for(unsigned int i=0; i<7; i++)
  { TH+=(THByte&1)*Temp;        //   
    TL+=(TLByte&1)*Temp;        //   
    THByte>>=1;  TLByte>>=1;    //    
    Temp*=2;                    //    (  )
  }
  //   
  if(THByte)    TH*=-1;         //      0,   
  if(TLByte)    TL*=-1;         //      0,   
  //      
  TempStr="TH="+IntToStr(TH)+" oC, TL="+IntToStr(TL)+" oC.";
  //     
  Form1->RichEdit9->Lines->Add(TempStr);       //   RichEdit9
  return;
}
//--------------------------------------------------------------------------- 