//---------------------------------------------------------------------------
#pragma hdrstop
#include "WireFunctions.h"      //    1-Wire
#include "PortFunctions.h"      //    
#include "MainProgram.h"        //    
#include "ProgressBar.h"        //   -
#include "CalcCRC.h"            //   CRC
#include "ChipFunctions.h"      //      
#include "FormFunctions.h"      //      
#include "SendMsgFunctions.h"   //     
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
//---        ------------------
//---------------------------------------------------------------------------
AnsiString GetDeviceDescription(unsigned char FamilyCode)
{ switch(FamilyCode)
  { case 0x1: return "DS2401/DS2411/DS1990A - Silicon Serial Number";				//  ,  ROM
    case 0x2: return "DS1991 - 3x384-bits Secure NVRAM";							// 
    case 0x4: return "DS2404/DS1994 - Time Chip with 4096 NVRAM";					// 
    case 0x5: return "DS2405 - Addressable Switch";									// 
    case 0x6: return "DS1993 - 4096-bits of Nonvolatile Memory";					// 
    case 0x8: return "DS1992 - 1024-bits of Nonvolatile Memory";					// 
    case 0x9: return "DS2502/DS1982 - 1024-bits EPROM (add only)";								//    +12
    case 0xA: return "DS1995 - 16384-bits of Nonvolatile Memory";					// 
    case 0xB: return "DS2505/DS1985 - 16384-bits EPROM (add only)";								//    +12
    case 0xC: return "DS1996 - 65536-bits of Nonvolatile Memory";					// 
    case 0xF: return "DS2506/DS1986 - 65536-bits EPROM (add only)";								//    +12
    case 0x10: return "DS18S20/DS1920 - Digital Thermometer";						// 
    case 0x12: return "DS2406/DS2407 - Dual Addr. Switch + 1Kb Memory";							//    +12
    case 0x14: return "DS2430A/DS1971 - 256-bits EEPROM + 64-bits OTP";         	// 
	case 0x18: return "DS1963S - 4096-bits Monetary Nonvolatile Memory with SHA-1";				//   (, reset low  540 ,   500 )
    case 0x1A: return "DS1963L - 4096-bits Monetary Nonvolatile Memory";			// 
    case 0x1B: return "DS2436 - Battery Id/Monitor Chip";								//   ( )
    case 0x1C: return "DS28E04-100 - 4096-bits EEPROM with PIO";								//   (, reset low  504 ,  640 )
    case 0x1D: return "DS2423 - 4096-bits SRAM";									//  (like as DS1963L, except tamper-bytes)
    case 0x1F: return "DS2409 - MicroLAN Coupler";
    case 0x20: return "DS2450 - Quad A/D Converter";
    case 0x21: return "DS1921 - Thermochron";													//   (, reset low  540 ,  640 )
    case 0x22: return "DS1822 - Digital Thermometer";								//  (same as DS18B20)
    case 0x23: return "DS2433 - 4096-bits EEPROM";                              	// 
    case 0x24: return "DS2415 - Time Chip";											// 
    case 0x26: return "DS2438 - Smart Battery Monitor";
    case 0x27: return "DS2417 - Time Chip with Interrupt";							// 
    case 0x28: return "DS18B20 - Digital Thermometer";								// 
    case 0x29: return "DS2408 - 8-channel Addressable Switch";									//   (, reset low  660 ,  720 )
    case 0x2C: return "DS2890 - Digital Potentiometer";								// 
    case 0x2D: return "DS2431 - 1024-bits EEPROM";                  	            // 
    case 0x2E: return "DS2770 - Battery Monitor & Charge Controller";
    case 0x30: return "DS2760/DS2761/DS2762 - Hi Prec. Li+ Battery Monitor";
    case 0x31: return "DS2720 - Addressable Single-Cell Lithium Protection IC";
    case 0x33: return "DS2432 - 1024-bits EEPROM with SHA1 Engine";
    case 0x36: return "DS2740 - Hi-Precision Coulomb Counter";
    case 0x37: return "DS1977 - Password Protected 32KB EEPROM";					// 
    case 0x3A: return "DS2413 - 2-channel Addresasble Switch";									//   (, reset low  600 ,   500 )
    case 0x41: return "DS2422/DS1922L/DS1922T/DS1923 - 8KB Temp./Datalogger";					//   (, reset low  690 ,  720 )
    case 0x51: return "DS2751 - Multichemistry Battery Fuel Gauge";
    case 0x81: return "DS1420 - Serial ID Button";									//  ,  ROM
    case 0x82: return "DS1425 - 3x384-bit Secure NVRAM";							//  (same as DS1991)
    case 0x84: return "DS2404S/DS1427 - EconoRAM (4096-bits NVRAM) + Time Chip";	//  (same as DS1994)
    case 0x89: return "DS2502-E48/DS2502-UNW/DS1982U - 48-bit Node Address Chip";				//    +12
    case 0x8B: return "DS2505-UNW/DS1985U - 16Kb EPROM (add only) UniqueWare";					//    +12
    case 0x8F: return "DS2506-UNW/DS1986U - 64Kb EPROM (add only) UniqueWare";					//    +12
    case 0xFF: return "LCD";
    default: return "Unknown Device";
  }
}
//---------------------------------------------------------------------------
//---      1-WIRE (0xF0) ----------------------------
//---------------------------------------------------------------------------
int SearchDevices()
{ //SEARCH DEVICES
  WRITE_BUFFER wr_buf;  //        
  READ_BUFFER rd_buf;   //        

  unsigned char CRC8, Temp, ROM[8];
  char buf[2], value[1], hexbuf[3];
  bool SlaveBit, ISlaveBit, LastDev;
  int Step, LastFork, ZeroFork, CurrentByte, CurrentBit;
  AnsiString TempStr;
  int  DevCounter;      //   

  //
  DevCounter=0; CRC8=0;
  Form1->StringGrid1->RowCount=DevCounter+2; //    
  Form1->StringGrid1->Cells[0][1]="";
  Form1->StringGrid1->Cells[1][1]="";
  Form1->StringGrid1->Cells[2][1]="";
  //    
  SendMsgToLogForm(9);    //   

  for(int i=0; i<8; i++)
  { ROM[i]=0; }  //  ROM
  LastDev=false;
  LastFork=-1;
  // -
  Application->CreateForm(__classid(TLBar),&LBar); //    -
  LBar->Show();                   //     

  while(!LastDev) //     
  { CRC8=0; ZeroFork=-1; Step=0;
    // Reset
    wr_buf.Size=1;
    wr_buf.Data[0]=0x60;
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);//   
                           LBar->Free();        //    -
                           ClosePort();
                           return DevCounter;
                         }
    Sleep(1);
    // Search ROM
    wr_buf.Size=2;
    wr_buf.Data[0]=0x61;     //SendByte_1W
    wr_buf.Data[1]=0xF0;     //Search ROM
    if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);//   
                           LBar->Free();        //    -
                           ClosePort();
                           return DevCounter;
                         }
    Sleep(1);
    while(Step<64)      //       (64 )
    { CurrentByte=Step/8; CurrentBit=Step-8*CurrentByte;
      //    
      wr_buf.Size=1;
      wr_buf.Data[0]=0x64; //   
      if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);//   
                             LBar->Free();        //    -
                             ClosePort();
                             return DevCounter;
                           }
      Sleep(1);
      // 
      rd_buf=ReadFrom();
      if(!rd_buf.Size){ SendMsgToLogForm(11);//   
                        LBar->Free();        //    -
                        ClosePort();
                        return DevCounter;
                      }
      if(rd_buf.Data[0]==0x0F) SlaveBit=false;
      else                     SlaveBit=true;
      //    
      wr_buf.Size=1;
      wr_buf.Data[0]=0x64; //   
      if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);//   
                             LBar->Free();        //    -
                             ClosePort();
                             return DevCounter;
                           }
      Sleep(1);
      // 
      rd_buf=ReadFrom();
      if(!rd_buf.Size){ SendMsgToLogForm(11);//   
                        LBar->Free();        //    -
                        ClosePort();
                        return DevCounter;
                      }
      if(rd_buf.Data[0]==0x0F) ISlaveBit=false;
      else                     ISlaveBit=true;
      //    ?
      if(SlaveBit==ISlaveBit)  //   -   
      { if (SlaveBit)          //  SlaveBit=1 ()
        { SendMsgToLogForm(12);//    2 
          LBar->Free();        //    -
          ClosePort();
          return DevCounter;
        }
        else                   //  SlaveBit=0 ()
        { if(Step==LastFork)
          { wr_buf.Data[1]=1; }     // MasterBit=1
          else
          { if(Step>LastFork)
            { wr_buf.Data[1]=0; }
            else
            { wr_buf.Data[1]=ROM[CurrentByte]>>CurrentBit; }
            if((wr_buf.Data[1]&1)==0)    //  MasterBit=0
            { ZeroFork=Step; }
          }
        }
      }
      else //    (  )
      { wr_buf.Data[1]=SlaveBit; }  // MasterBit=SlaveBit

      //    ROM
      Temp=1<<CurrentBit;                     //  =1
      Temp=~Temp;                             //  (  = 1,  )
      ROM[CurrentByte]=ROM[CurrentByte]&Temp; //     ROM
      Temp=(wr_buf.Data[1]&1)<<CurrentBit;    //  =MasterBit (     )
      ROM[CurrentByte]=ROM[CurrentByte]|Temp; //  MasterBit
      // CRC8
      CRC8=CalcCRC8(CRC8, wr_buf.Data[1]);
      //   
      wr_buf.Size=2;
      wr_buf.Data[0]=0x63;
      if(!WriteTo(wr_buf)) { SendMsgToLogForm(11);//   
                             LBar->Free();        //    -
                             ClosePort();
                             return DevCounter;
                           }
      Sleep(1);
      Step=Step+1;
      //   -
      LBar->ProgressBar1->Position=100*Step/64; //    -
    }
    //   
    LastFork=ZeroFork;
    if(LastFork<0)
     LastDev=true;
    if(CRC8==0)
    { DevCounter=DevCounter+1;                    //    
      Form1->StringGrid1->RowCount=DevCounter+1;  //    
      TempStr="";
      for(int i=0; i<8; i++)
      {  value[0]=ROM[i];
         BinToHex(value,hexbuf,1);       //   
         hexbuf[2]=0;                    //   
         TempStr=TempStr+hexbuf+" ";
      }
      Form1->StringGrid1->Cells[0][DevCounter]=IntToStr(DevCounter);
      Form1->StringGrid1->Cells[1][DevCounter]=TempStr;
      TempStr=GetDeviceDescription(ROM[0]);             //    
      Form1->StringGrid1->Cells[2][DevCounter]=TempStr; //      
      Form1->StringGrid1->ColWidths[2]=394;             //    
      SendMsgToLogForm(13);// CRC8 Ok
    }
    else
    { SendMsgToLogForm(14);//  CRC8
      LBar->Free();      //    -
      ClosePort();
      return DevCounter;
    }
  }
  //*************************************************************************
  LBar->Free();         //    -
  SendMsgToLogForm(10); //    
  return DevCounter;    //  -   
}
//---------------------------------------------------------------------------
